from typing import Any

from asgiref.testing import ApplicationCommunicator as BaseApplicationCommunicator

def no_op() -> None: ...

class ApplicationCommunicator(BaseApplicationCommunicator):
    async def send_input(self, message: dict[str, Any]) -> None: ...
    async def receive_output(self, timeout: float = ...) -> dict[str, Any]: ...
    async def receive_nothing(self, timeout: float = ..., interval: float = ...) -> bool: ...
    async def wait(self, timeout: float = ...) -> None: ...
    def stop(self, exceptions: bool = ...) -> None: ...



from typing import Any

from asgiref.typing import WebSocketConnectEvent, WebSocketDisconnectEvent, WebSocketReceiveEvent
from channels.consumer import AsyncConsumer, SyncConsumer, _ChannelScope
from channels.layers import BaseChannelLayer

class WebsocketConsumer(SyncConsumer):
    groups: list[str] | None
    scope: _ChannelScope
    channel_name: str
    channel_layer: BaseChannelLayer
    channel_receive: Any
    base_send: Any

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def websocket_connect(self, message: WebSocketConnectEvent) -> None: ...
    def connect(self) -> None: ...
    def accept(self, subprotocol: str | None = ..., headers: list[tuple[str, str]] | None = ...) -> None: ...
    def websocket_receive(self, message: WebSocketReceiveEvent) -> None: ...
    def receive(self, text_data: str | None = ..., bytes_data: bytes | None = ...) -> None: ...
    def send(  # type: ignore[override]
        self, text_data: str | None = ..., bytes_data: bytes | None = ..., close: bool = ...
    ) -> None: ...
    def close(self, code: int | bool | None = ..., reason: str | None = ...) -> None: ...
    def websocket_disconnect(self, message: WebSocketDisconnectEvent) -> None: ...
    def disconnect(self, code: int) -> None: ...

class JsonWebsocketConsumer(WebsocketConsumer):
    def receive(self, text_data: str | None = ..., bytes_data: bytes | None = ..., **kwargs: Any) -> None: ...
    def receive_json(self, content: Any, **kwargs: Any) -> None: ...
    def send_json(self, content: Any, close: bool = ...) -> None: ...
    @classmethod
    def decode_json(cls, text_data: str) -> Any: ...
    @classmethod
    def encode_json(cls, content: Any) -> str: ...

class AsyncWebsocketConsumer(AsyncConsumer):
    groups: list[str] | None
    scope: _ChannelScope
    channel_name: str
    channel_layer: BaseChannelLayer
    channel_receive: Any
    base_send: Any

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    async def websocket_connect(self, message: WebSocketConnectEvent) -> None: ...
    async def connect(self) -> None: ...
    async def accept(self, subprotocol: str | None = ..., headers: list[tuple[str, str]] | None = ...) -> None: ...
    async def websocket_receive(self, message: WebSocketReceiveEvent) -> None: ...
    async def receive(self, text_data: str | None = ..., bytes_data: bytes | None = ...) -> None: ...
    async def send(  # type: ignore[override]
        self, text_data: str | None = ..., bytes_data: bytes | None = ..., close: bool = ...
    ) -> None: ...
    async def close(self, code: int | bool | None = ..., reason: str | None = ...) -> None: ...
    async def websocket_disconnect(self, message: WebSocketDisconnectEvent) -> None: ...
    async def disconnect(self, code: int) -> None: ...

class AsyncJsonWebsocketConsumer(AsyncWebsocketConsumer):
    async def receive(self, text_data: str | None = ..., bytes_data: bytes | None = ..., **kwargs: Any) -> None: ...
    async def receive_json(self, content: Any, **kwargs: Any) -> None: ...
    async def send_json(self, content: Any, close: bool = ...) -> None: ...
    @classmethod
    async def decode_json(cls, text_data: str) -> Any: ...
    @classmethod
    async def encode_json(cls, content: Any) -> str: ...



from collections.abc import Awaitable
from typing import Any, ClassVar, Protocol, type_check_only

from asgiref.typing import ASGIReceiveCallable, ASGISendCallable, Scope, WebSocketScope
from channels.auth import UserLazyObject
from channels.db import database_sync_to_async
from channels.layers import BaseChannelLayer
from django.contrib.sessions.backends.base import SessionBase
from django.utils.functional import LazyObject

@type_check_only
class _LazySession(SessionBase, LazyObject):  # type: ignore[misc]
    _wrapped: SessionBase

# Base ASGI Scope definition
@type_check_only
class _ChannelScope(WebSocketScope, total=False):
    # Channel specific
    channel: str
    url_route: dict[str, Any]
    path_remaining: str

    # Auth specific
    cookies: dict[str, str]
    session: _LazySession
    user: UserLazyObject | None

def get_handler_name(message: dict[str, Any]) -> str: ...
@type_check_only
class _ASGIApplicationProtocol(Protocol):
    consumer_class: Any
    consumer_initkwargs: dict[str, Any]

    def __call__(self, scope: Scope, receive: ASGIReceiveCallable, send: ASGISendCallable) -> Awaitable[None]: ...

class AsyncConsumer:
    _sync: ClassVar[bool] = ...
    channel_layer_alias: ClassVar[str] = ...

    scope: _ChannelScope
    channel_layer: BaseChannelLayer
    channel_name: str
    channel_receive: ASGIReceiveCallable
    base_send: ASGISendCallable

    async def __call__(self, scope: _ChannelScope, receive: ASGIReceiveCallable, send: ASGISendCallable) -> None: ...
    async def dispatch(self, message: dict[str, Any]) -> None: ...
    async def send(self, message: dict[str, Any]) -> None: ...
    @classmethod
    def as_asgi(cls, **initkwargs: Any) -> _ASGIApplicationProtocol: ...

class SyncConsumer(AsyncConsumer):
    _sync: ClassVar[bool] = ...

    @database_sync_to_async
    def dispatch(self, message: dict[str, Any]) -> None: ...  # type: ignore[override]
    def send(self, message: dict[str, Any]) -> None: ...  # type: ignore[override]
